@@ -1,12 +1,21 @@
 package org.gcube.portal.databook.server;
 
-import java.util.Date;
+import java.text.SimpleDateFormat;
+import java.util.Calendar;
+import java.util.Collections;
+import java.util.Comparator;
+import java.util.HashMap;
+import java.util.LinkedHashMap;
+import java.util.LinkedList;
 import java.util.List;
-import java.util.UUID;
+import java.util.Map;
+import java.util.Map.Entry;
 
-import org.gcube.portal.databook.shared.Invite;
-import org.gcube.portal.databook.shared.InviteOperationResult;
-import org.gcube.portal.databook.shared.InviteStatus;
+import org.gcube.portal.databook.shared.Feed;
+import org.gcube.portal.databook.shared.ex.ColumnNameNotFoundException;
+import org.gcube.portal.databook.shared.ex.FeedIDNotFoundException;
+import org.gcube.portal.databook.shared.ex.FeedTypeNotFoundException;
+import org.gcube.portal.databook.shared.ex.PrivacyLevelTypeNotFoundException;
 import org.junit.AfterClass;
 import org.junit.BeforeClass;
 import org.junit.Test;
@@ -23,35 +32,204 @@ public class DatabookCassandraTest {
 	public static void close(){
 		store.closeConnection();
 		System.out.println("End");
-	}
-
-	@Test
-	public void testInvites()  {
-		String controlCode = UUID.randomUUID().toString();
-		String vreid = "/gcube/devsec/devVRE";
-		String email = "ciccio@gmail.com";
-		
-		InviteStatus statusToLookfor = InviteStatus.ACCEPTED;
-		
-		Invite invite = new Invite(UUID.randomUUID().toString(), "andrea.rossi", vreid, email, controlCode, InviteStatus.PENDING, new Date(), "Andrea Rossi");
-		try {
-			InviteOperationResult result = store.saveInvite(invite);
-			System.out.println(result);
-			String inviteid = store.isExistingInvite(vreid, email);
-//			System.out.println(store.readInvite(inviteid));
-//			store.setInviteStatus(vreid, email, InviteStatus.ACCEPTED);
-//			System.out.println(store.readInvite(inviteid));
-			System.out.println("Looking for all invites in " + vreid + " with status " + statusToLookfor);
-			List<Invite> invites = store.getInvitedEmailsByVRE(vreid, statusToLookfor, InviteStatus.PENDING);
-			for (Invite invite2 : invites) {
-				System.out.println(invite2);
-			}
-		} catch (Exception e) {
-			e.printStackTrace();
-		}
 
 	}
 
+	//	@Test
+	//	public void getHashTagsFilteredByTime() throws PrivacyLevelTypeNotFoundException, FeedTypeNotFoundException, FeedIDNotFoundException, ColumnNameNotFoundException{
+	//
+	//		int windowSize = 6; // go back windowSize months
+	//
+	//		String vreUnderTest = "/gcube/devsec/devVRE";
+	//
+	//		// reference time 
+	//		Calendar calendar = Calendar.getInstance();
+	//		int currentMonth = calendar.get(Calendar.MONTH); // jan = 0, ..... dec = 11
+	//		calendar.set(Calendar.MONTH, currentMonth -  windowSize); // the year is automatically decreased if needed
+	//		SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
+	//		System.out.println("Reference time for trending topics is " + format.format(calendar.getTime()));
+	//
+	//		Map<String, Integer> res = 
+	//				store.getVREHashtagsWithOccurrenceFilteredByTime(
+	//						vreUnderTest, calendar.getTimeInMillis());
+	//
+	//		// find max score inside the list (counter)
+	//		int max = 0;
+	//		for(Entry<String, Integer> entry : res.entrySet()){
+	//
+	//			max = max < entry.getValue() ? entry.getValue() : max;
+	//
+	//		}
+	//
+	//		// normalize
+	//		Map<String, Double> normalized = new HashMap<String, Double>();
+	//		for(Entry<String, Integer> entry : res.entrySet()){
+	//
+	//			normalized.put(entry.getKey(), (double)entry.getValue() / (double)max);
+	//
+	//		}
+	//
+	//		// create the weight for each entry as:
+	//		// w = 0.6 * normalized_score + 0.4 * freshness
+	//		// freshness is evaluated as (window_size - latest_feed_for_hashtag_in_window_month)/window_size
+	//		Map<String, Double> scoredList = new HashMap<String, Double>();
+	//		for(Entry<String, Integer> entry : res.entrySet()){
+	//
+	//			double weight = 0.6 * normalized.get(entry.getKey());
+	//
+	//			// retrieve the last feed for this hashtag and locate it into the window
+	//			List<Feed> mostRecentFeedForHashtag = store.getVREFeedsByHashtag(vreUnderTest, entry.getKey());
+	//
+	//			// retrieve the most recent one among these feeds
+	//			Collections.sort(mostRecentFeedForHashtag, Collections.reverseOrder());
+	//
+	//			// locate into the window
+	//			Calendar locateInWindow = Calendar.getInstance();
+	//			locateInWindow.setTimeInMillis(mostRecentFeedForHashtag.get(0).getTime().getTime());
+	//
+	//			// get the month
+	//			int sub = currentMonth - locateInWindow.get(Calendar.MONTH);
+	//			int value = sub >= 0? sub : 12 - Math.abs(sub);
+	//			double freshness = (double)(windowSize - value) / (double)(windowSize);
+	//			System.out.println("freshness is " + freshness + " because the last feed has month " + locateInWindow.get(Calendar.MONTH));
+	//
+	//			weight += 0.4 * freshness;
+	//
+	//			scoredList.put(entry.getKey(), weight);
+	//		}
+	//
+	//		// print sorted
+	//		Map<String, Double> scoredListSorted = sortByValue(scoredList);
+	//		for(Entry<String, Double> entry : scoredListSorted.entrySet()){
+	//
+	//			System.out.println("[hashtag=" + entry.getKey() + " , weight="  + entry.getValue() + "]");
+	//		}
+	//	}
+	//
+	//	public static <K, V extends Comparable<? super V>> Map<K, V> 
+	//	sortByValue( Map<K, V> map )
+	//	{
+	//		List<Map.Entry<K, V>> list =
+	//				new LinkedList<Map.Entry<K, V>>( map.entrySet() );
+	//		Collections.sort( list, new Comparator<Map.Entry<K, V>>()
+	//				{
+	//			public int compare( Map.Entry<K, V> o1, Map.Entry<K, V> o2 )
+	//			{
+	//				return (o2.getValue()).compareTo( o1.getValue() );
+	//			}
+	//				});
+	//
+	//		Map<K, V> result = new LinkedHashMap<K, V>();
+	//		for (Map.Entry<K, V> entry : list)
+	//		{
+	//			result.put( entry.getKey(), entry.getValue() );
+	//		}
+	//		return result;
+	//	}
+
+
+	//	@Test
+	//	public void getHashTags() throws PrivacyLevelTypeNotFoundException, FeedTypeNotFoundException, FeedIDNotFoundException, ColumnNameNotFoundException{
+	//		List<Feed> resList = store.getVREFeedsByHashtag("/gcube/devsec/devVRE", "#test");
+	//
+	//		for (Feed feed : resList) {
+	//			System.out.println(feed.getTime());
+	//		}
+	//
+	//	}
+
+	//	@Test
+	//	public void getComment(){
+	//
+	//		String uuid = "820969b2-4632-4197-9fd6-5aafab781faa";
+	//
+	//		Comment c;
+	//		try {
+	//			c = store.readCommentById(uuid);
+	//			System.err.println(c);
+	//		} catch (CommentIDNotFoundException e) {
+	//			// TODO Auto-generated catch block
+	//			System.err.println(e.toString());
+	//		}
+	//	}
+
+	//	@Test
+	//	public void vreIds(){
+	//		
+	//		try {
+	//			List<String> ids = store.getAllVREIds();
+	//			System.out.println(ids);
+	//		} catch (ConnectionException e) {
+	//			// TODO Auto-generated catch block
+	//			e.printStackTrace();
+	//		}
+	//		
+	//	}
+
+	//	@Test
+	//	public void testFeedNumberPerUser() {
+	//		String userid = "massimiliano.assante";
+	//		
+	//		List<Feed> feeds = null;
+	//		int numComment = 0;
+	//		long init = System.currentTimeMillis();
+	//		try {
+	//			feeds = store.getAllFeedsByUser(userid);
+	//			
+	//			for (Feed feed : feeds) {
+	//				List<Comment> comments = store.getAllCommentByFeed(feed.getKey());
+	//				
+	//				
+	//				for (Comment comment : comments) {
+	//					 numComment ++;
+	//				}
+	//			}
+	//			
+	//		} catch (PrivacyLevelTypeNotFoundException | FeedTypeNotFoundException
+	//				| ColumnNameNotFoundException | FeedIDNotFoundException e) {
+	//			// TODO Auto-generated catch block
+	//			System.err.println(e.toString());
+	//		}
+	//		long end = System.currentTimeMillis();
+	//		System.err.println("retrieved " + feeds.size() + " and " + numComment + " in " + (end - init) + "ms");
+	//	}
+
+	//		@Test
+	//		public void testAttachments() {
+	//			Attachment a1 = new Attachment(UUID.randomUUID().toString(), "www1", "gattino1", "description1", "http://cdn.tuttozampe.com/wp-content/uploads/2010/09/ipoglicemia-gatto.jpg", "image/jpg");
+	//			Attachment a2 = new Attachment(UUID.randomUUID().toString(), "www2", "name2", "description2", "http://www.gcomegatto.it/wp-content/uploads/2015/01/09gatto.jpg","image/jpg");
+	//			Attachment a3 = new Attachment(UUID.randomUUID().toString(), "www3", "name3", "description3", "http://cdn.tuttozampe.com/wp-content/uploads/2010/09/ipoglicemia-gatto.jpg","image/jpg");
+	//			List<Attachment> toPass = new ArrayList<Attachment>();
+	//			toPass.add(a1);
+	//			toPass.add(a2);
+	//			toPass.add(a3);
+	//		
+	//			String feedId = UUID.randomUUID().toString();
+	//			Feed feed = new Feed(feedId, FeedType.TWEET, "massimiliano.assante", new Date(), "/gcube/devsec/devVRE", 
+	//					"http://www.dailybest.it/wp-content/uploads/2015/10/gattini-nele-ciotole-e1344352237289.jpg", 
+	//					"http://www.dailybest.it/wp-content/uploads/2015/10/gattini-nele-ciotole-e1344352237289.jpg", 
+	//					"This post has attachments (gattini) ", PrivacyLevel.SINGLE_VRE, 
+	//					"Massimiliano Assante", 
+	//					"massimiliano.assante@isti.cnr.it", 
+	//					"http://www.dailybest.it/wp-content/uploads/2015/10/gattini-nele-ciotole-e1344352237289.jpg", 
+	//					"Gattino", 
+	//					"linkDesc", 
+	//					"image/jpeg", false);
+	//			feed.setMultiFileUpload(true);
+	//			assertTrue(store.saveUserFeed(feed, toPass));
+	//			System.out.println("Wrote post? ");
+	//			System.out.println("Feed has the following attachments: ");
+	//			try {
+	//				for (Attachment at : store.getAttachmentsByFeedId(feedId)) {
+	//					System.out.println(at);
+	//				}
+	//			} catch (FeedIDNotFoundException e) {
+	//				// TODO Auto-generated catch block
+	//				e.printStackTrace();
+	//			}
+	//			
+	//		}
+
 	//	@Test
 	//	public void testHashTag() {
 	//		try {	
@@ -89,59 +267,56 @@ public class DatabookCassandraTest {
 
 
 	//	/**
-	//	 * use exclusively to add a new (Static) CF to a keyspace
+	//	 * use exclusively to add a new (Static) CF to a keyspace with a secondary index
 	//	 */
 	//	@Test
-	//	public void addInvitesStaticColumnFamilies() {
-	//		ColumnFamily<String, String> CF_INVITES = ColumnFamily.newColumnFamily(DBCassandraAstyanaxImpl.INVITES, StringSerializer.get(), StringSerializer.get());
+	//	public void addAttachmentStaticColumnFamilies() {
+	//		ColumnFamily<String, String> CF_ATTACHMENTS = ColumnFamily.newColumnFamily(DBCassandraAstyanaxImpl.ATTACHMENTS, StringSerializer.get(), StringSerializer.get());
 	//
 	//		try {
-	//			new CassandraClusterConnection(false).getKeyspace().createColumnFamily(CF_INVITES, ImmutableMap.<String, Object>builder()
-	//			        .put("default_validation_class", "UTF8Type")
-	//			        .put("key_validation_class",     "UTF8Type")
-	//			        .put("comparator_type",          "UTF8Type")
-	//			        .build());
+	//			String colNameToIndex = "feedId";
+	//			new CassandraClusterConnection(false).getKeyspace().createColumnFamily(CF_ATTACHMENTS, ImmutableMap.<String, Object>builder()
+	//							.put("column_metadata", ImmutableMap.<String, Object>builder()
+	//								.put(colNameToIndex, ImmutableMap.<String, Object>builder()
+	//									.put("validation_class", "UTF8Type")
+	//									.put("index_name",       "FeedIndex_"+UUID.randomUUID().toString().substring(0,5))
+	//									.put("index_type",       "KEYS")
+	//									.build())
+	//								.build())
+	//							.build());
+	//
 	//
 	//		} catch (ConnectionException e) {
 	//			e.printStackTrace();
 	//		}
-	//		System.out.println("addInvitesStaticColumnFamily END");
+	//		System.out.println("addStaticColumnFamily END");
 	//	}
 
 
-	//	/**
-	//	 * use exclusively to add a new (Dynamic) CF to a keyspace
-	//	 */
-	//	@Test
-	//	public void addInvitesDynamicColumnFamilies() {
-	//		System.out.println("addInvitesColumnFamilies");
-	//		ColumnFamily<String, String> cf_HashtagsCounter =	new ColumnFamily<String, String>(
-	//				DBCassandraAstyanaxImpl.VRE_INVITES,        // Column Family Name
-	//				StringSerializer.get(),   // Key Serializer
-	//				StringSerializer.get());  // Column Serializer
-	//		ColumnFamily<String, String> cf_HashtagTimeline =	new ColumnFamily<String, String>(
-	//				DBCassandraAstyanaxImpl.EMAIL_INVITES,  // Column Family Name
-	//				StringSerializer.get(),   // Key Serializer
-	//				StringSerializer.get());  // Column Serializer
-	//
-	//		try {
-	//			new CassandraClusterConnection(false).getKeyspace().createColumnFamily(cf_HashtagsCounter, ImmutableMap.<String, Object>builder()
-	//					.put("default_validation_class", "UTF8Type")
-	//					.put("key_validation_class",     "UTF8Type")
-	//					.put("comparator_type",          "UTF8Type")
-	//					.build());
+	//		/**
+	//		 * use exclusively to add a new (Dynamic) CF to a keyspace
+	//		 */
+	//		@Test
+	//		public void addInvitesDynamicColumnFamilies() {
+	//			System.out.println("UserNotificationsUnread");
+	//			ColumnFamily<String, String> cf_UserNotificationsUnreadTimeline =	new ColumnFamily<String, String>(
+	//					DBCassandraAstyanaxImpl.USER_NOTIFICATIONS_UNREAD,  // Column Family Name
+	//					StringSerializer.get(),   // Key Serializer
+	//					StringSerializer.get());  // Column Serializer
+	//	
+	//			try {
 	//			
-	//			new CassandraClusterConnection(false).getKeyspace().createColumnFamily(cf_HashtagTimeline, ImmutableMap.<String, Object>builder()
-	//					.put("default_validation_class", "UTF8Type")
-	//					.put("key_validation_class",     "UTF8Type")
-	//					.put("comparator_type",          "UTF8Type")
-	//					.build());
-	//			
-	//		} catch (ConnectionException e) {
-	//			e.printStackTrace();
+	//				new CassandraClusterConnection(false).getKeyspace().createColumnFamily(cf_UserNotificationsUnreadTimeline, ImmutableMap.<String, Object>builder()
+	//						.put("default_validation_class", "UTF8Type")
+	//						.put("key_validation_class",     "UTF8Type")
+	//						.put("comparator_type",          "UTF8Type")
+	//						.build());
+	//				
+	//			} catch (ConnectionException e) {
+	//				e.printStackTrace();
+	//			}
+	//			System.out.println("UserNotificationsUnread END");
 	//		}
-	//		System.out.println("addInvitesColumnFamilies END");
-	//	}
 
 
 	//	private List<String> getKeys() {
@@ -446,36 +621,41 @@ public class DatabookCassandraTest {
 	//
 	//
 	//
-	//	@Test
-	//	public void testComments() {
-	//		int count = 10;
-	//		Feed feed = new Feed(UUID.randomUUID().toString(), FeedType.SHARE, "massimiliano.assante", new Date(), "/gcube/devsec/devVRE", 
-	//				"http://www.d4science.org/monitor", "thumbUri", "This is feed that is going to be commented ", PrivacyLevel.PUBLIC, "Massimiliano Assante", 
-	//				"massimiliano.assante@isti.cnr.it", "thumburl", "linkTitle", "linkDesc", "host", false);
-	//		assertTrue(store.saveUserFeed(feed));
-	//
-	//		Comment toDelete = null;
-	//		for (int i = 0; i < count; i++) {
+	//		@Test
+	//		public void testComments() {
+	//			int count = 10;
+	//			Feed feed = new Feed(UUID.randomUUID().toString(), FeedType.SHARE, "massimiliano.assante", new Date(), "/gcube/devsec/devVRE", 
+	//					"http://www.d4science.org/monitor", "thumbUri", "This is feed that is going to be commented ", PrivacyLevel.PUBLIC, "Massimiliano Assante", 
+	//					"massimiliano.assante@isti.cnr.it", "thumburl", "linkTitle", "linkDesc", "host", false);
+	//			assertTrue(store.saveUserFeed(feed));
+	//	
+	//			Comment toDelete = null;
+	//			for (int i = 0; i < count; i++) {
+	//				try {
+	//					toDelete = new Comment(UUID.randomUUID().toString(),"leonardo.candela", 
+	//							new Date(), feed.getKey().toString(), "This comment #"+i, "Leonardo Candela", "thumbUrl");
+	//					assertTrue(store.addComment(toDelete));
+	//	
+	//				} catch (FeedIDNotFoundException e) {
+	//					System.out.println("Exception feed id not found");
+	//				}			
+	//			}
+	//			System.out.println("GetAllCOmmentsByFeed ");
+	//			for (Comment cm : store.getAllCommentByFeed(feed.getKey().toString())) {
+	//				try {
+	//					System.out.println(store.readCommentById(cm.getKey()));
+	//				} catch (CommentIDNotFoundException e) {
+	//					// TODO Auto-generated catch block
+	//					e.printStackTrace();
+	//				}
+	//			};
+	//	
 	//			try {
-	//				toDelete = new Comment(UUID.randomUUID().toString(),"leonardo.candela", 
-	//						new Date(), feed.getKey().toString(), "This comment #"+i, "Leonardo Candela", "thumbUrl");
-	//				assertTrue(store.addComment(toDelete));
-	//
-	//			} catch (FeedIDNotFoundException e) {
+	//				assertTrue(store.deleteComment(toDelete.getKey(), toDelete.getFeedid()));
+	//			} catch (Exception e) {
 	//				System.out.println("Exception feed id not found");
-	//			}			
+	//			}
 	//		}
-	//		System.out.println("GetAllCOmmentsByFeed ");
-	//		for (Comment cm : store.getAllCommentByFeed(feed.getKey().toString())) {
-	//			System.out.println(cm.getText());
-	//		};
-	//
-	//		try {
-	//			assertTrue(store.deleteComment(toDelete.getKey(), toDelete.getFeedid()));
-	//		} catch (Exception e) {
-	//			System.out.println("Exception feed id not found");
-	//		}
-	//	}
 
 
 }
