@@ -9,7 +9,18 @@ package org.gcube.portal.databook.shared;
  * introduced due to urgent matters
  */
 public enum NotificationType {
-
+	/**
+	 * use to notify a user he got a Tabular Resource shared
+	 */
+	TDM_TAB_RESOURCE_SHARE,
+	/**
+	 * use to notify a user he got a TDM Rule shared
+	 */
+	TDM_RULE_SHARE,
+	/**
+	 * use to notify a user he got a TDM Templated shared
+	 */
+	TDM_TEMPLATE_SHARE,
 	/**
 	 * use to notify a user he got a workspace folder shared
 	 */
@@ -18,6 +29,14 @@ public enum NotificationType {
 	 * use to notify a user that a user in the share unshared
 	 */
 	WP_FOLDER_UNSHARE,
+	/**
+	 * use to notify a user that he got upgraded to administrator of a shared folder
+	 */
+	WP_ADMIN_UPGRADE,
+	/**
+	 * use to notify a user that he got downgraded from administrator of a shared folder
+	 */
+	WP_ADMIN_DOWNGRADE,
 	/**
 	 * use to notify a user that a new user was added in on of his workspace shared folder
 	 */
